/*
    Copyright (C) 2005-2008  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "nntpgrab_automation.h"
#include "nntpgrab_utils.h"
#include <glib/gi18n.h>

#if GLIB_CHECK_VERSION(2,16,0)
#include <gio/gio.h>

static GFileMonitor *monitor = NULL;

static void
directory_monitor_changed (GFileMonitor *monitor, GFile *file, GFile *other_file, GFileMonitorEvent event_type, gpointer user_data)
{
    char *filename;
    char *contents = NULL;
    GError *err = NULL;
    NZB *nzbfile;
    char *errmsg = NULL;
    ConfigOpts opts;

    if (event_type != G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT) {
        return;
    }

    if (!g_file_load_contents(file, NULL, &contents, NULL, NULL, &err)) {
        char *path = g_file_get_path(file);
        g_warning(__FILE__ ":%i Error occured while reading file\n%s:\n%s", __LINE__, path, err->message);

        g_error_free(err);
        g_free(path);

        return;
    }

    filename = g_file_get_basename(file);
    nntpgrab_utils_strip_nzb_extension(filename);

    nzbfile = nntpgrab_utils_parse_nzb_file(contents, &errmsg);
    if (!nzbfile) {
        if (!errmsg) {
            errmsg = g_strdup(_("Error occured while reading NZB File"));
        }

        g_warning(errmsg);
        g_free(errmsg);
        g_free(contents);
        g_free(filename);

        return;
    }

    nntpgrab_automation_import_nzb_file((NNTPGrabCore*) user_data, nzbfile, filename, &errmsg);
    if (errmsg) {
        g_warning(errmsg);
        g_free(errmsg);
    }

    g_free(contents);
    g_free(filename);

    opts = nntpgrab_config_get_opts((NNTPGrabCore*) user_data);
    if (opts.move_file_after_auto_import) {
        GFile *dest;
        char *path_orig;
        char *basename_orig;
        char *dirname_orig;
        char *dirname;
        char *path;

        path_orig = g_file_get_path(file);
        dirname_orig = g_path_get_dirname(path_orig);
        dirname = g_build_filename(dirname_orig, "Imported", NULL);
        g_mkdir_with_parents(dirname, 0700);

        basename_orig = g_file_get_basename(file);
        path = g_build_filename(dirname, basename_orig, NULL);

        dest = g_file_new_for_path(path);
        g_file_move(file, dest, G_FILE_COPY_BACKUP, NULL, NULL, NULL, NULL);

        g_free(path);
        g_free(dirname);
        g_free(dirname_orig);
        g_free(basename_orig);
        g_free(path_orig);
        g_object_unref(dest);
    } else {
        g_file_delete(file, NULL, NULL);
    }
}

gboolean
nntpgrab_automation_enable_auto_import(NNTPGrabCore *core, const char *directory, char **errmsg)
{
    if (monitor) {
        nntpgrab_utils_cancel_directory_monitor(monitor);
    }

    monitor = nntpgrab_utils_monitor_directory(directory, errmsg);
    if (!monitor) {
        return FALSE;
    }

    g_signal_connect(monitor, "changed", G_CALLBACK(directory_monitor_changed), core);

    return TRUE;
}

void
nntpgrab_automation_disable_auto_import(void)
{
    if (monitor) {
        nntpgrab_utils_cancel_directory_monitor(monitor);
        monitor = NULL;
    }
}

#endif
